import WO_full
import numpy as np
import tensorflow as tf
import matplotlib.pyplot as plt
from matplotlib import patheffects
import trieste
from trieste.space import Box
from trieste.data import Dataset
from trieste.models.gpflow import build_gpr, GaussianProcessRegression
from trieste.acquisition.combination import Product
from trieste.acquisition.rule import EfficientGlobalOptimization

def WOBO_model(seedVal, initPoints, numSteps):
    np.random.seed(seedVal)
    tf.random.set_seed(seedVal)

    inst_WO = WO_full.model()

    d_val = np.array([1.5, 1])

    search_space = Box([0.14, 294], [4, 355])

    OBJECTIVE = "OBJECTIVE"
    CONSTRAINT = "CONSTRAINT"


    def observer(query_points):
        obj = []
        con = []
        for qpoint in query_points:
            fval = inst_WO.ss_output(qpoint.numpy(), d_val)
            obj.append([float(fval['J'])])
            con.append([float(fval['xz'][5])-0.105])
        obj = tf.constant(obj, dtype=np.float64)
        con = tf.constant(con, dtype=np.float64)
        #print('observed')
        return {
            OBJECTIVE: Dataset(query_points, obj),
            CONSTRAINT: Dataset(query_points, con),
        }

    def create_bo_model(data):
        gpr = build_gpr(data, search_space, likelihood_variance=1e-7)
        return GaussianProcessRegression(gpr)

    num_initial_points = initPoints
    initial_data = observer(search_space.sample(num_initial_points))
    initial_models = trieste.utils.map_values(create_bo_model, initial_data)

    pof = trieste.acquisition.ProbabilityOfFeasibility(threshold=0)

    eci = trieste.acquisition.ExpectedConstrainedImprovement(
        OBJECTIVE, pof.using(CONSTRAINT)
    )
    rule = EfficientGlobalOptimization(eci) # type: ignore

    num_steps = numSteps
    bo = trieste.bayesian_optimizer.BayesianOptimizer(
        observer, search_space
    )

    data = bo.optimize(
        num_steps, initial_data, initial_models, rule, track_state=False
    ).try_get_final_datasets()

    mask_fail = (
        data[CONSTRAINT].observations.numpy().flatten().astype(float) < 0
    )

    resultX = []
    resultY = []
    conList = []

    for i in range(mask_fail.size):
        Xval = data[OBJECTIVE].query_points.numpy()[i]
        Yval = data[OBJECTIVE].observations.numpy()[i]
        gVal = data[CONSTRAINT].observations.numpy()[i]
        if not mask_fail[i]:
            resultX.append(Xval)
            resultY.append(Yval)
            conList.append(gVal)

    resultX = [x for _, x in sorted(zip(resultY, resultX))]
    conList = [x for _, x in sorted(zip(resultY, conList))]
    resultY = sorted(resultY)

    return resultX, conList, resultY

#resultX, conList, resultY = WOBO_model(1404,10,15)

#print('From B-Opt:')
#print(resultY[0])
#print(resultX[0])
#print('g: ', conList[0])