from casadi import *
from packages.model_tools import DynModel

class model(DynModel):
    def __init__(self):
        super().__init__()
        tunit = 3600;

        pP = 1043.38
        pE = 20.92
        pA = 79.23
        pB = 118.34

        xa = MX.sym('xa',1);
        xb = MX.sym('xb',1);
        xc = MX.sym('xc',1);
        xp = MX.sym('xp',1);
        xe = MX.sym('xe',1);
        xg = MX.sym('xg',1);

        Fa = MX.sym('Fa',1);
        Fb = MX.sym('Fb',1);
        Tr = MX.sym('Tr',1);

        Vr = 2105; # 2.63; # Reactor mass holdup - constant
        k01 = MX.sym('k01'); # 1.6599e6;
        # k02 = MX.sym('k02'); # 7.2117e8;
        # k03 = MX.sym('k03'); # 2.6745e12;
        # dP = MX.sym('dP');

        B1 = 6666.7; # degK
        B2 = 8333.3;
        B3 = 11111;

        k1 = 1.6599e6*exp(-B1/(Tr))*k01;
        k2 = 7.2117e8*exp(-B2/(Tr))#*k02;
        k3 = 2.6745e12*exp(-B3/(Tr))#*k03;

        dxa = tunit*((Fa - (Fa+Fb)*xa - Vr*xa*xb*k1)/Vr);
        dxb = tunit*((Fb - (Fa+Fb)*xb - Vr*xa*xb*k1 - Vr*xb*xc*k2)/Vr);
        dxc = tunit*(-(Fa+Fb)*xc/Vr + 2*xa*xb*k1 - 2*xb*xc*k2 - xc*xp*k3);
        dxp = tunit*(-(Fa+Fb)*xp/Vr + xb*xc*k2 - 0.5*xp*xc*k3);
        dxe = tunit*(-(Fa+Fb)*xe/Vr + 2*xb*xc*k2);
        dxg = tunit*(-(Fa+Fb)*xg/Vr + 1.5*xp*xc*k3);

        diff = vertcat(dxa,dxb,dxc,dxp,dxe,dxg);
        x_var = vertcat(xa,xb,xc,xp,xe,xg);
        d_var = vertcat(
            Fa,
            k01,
            #k02,
            #k03,
            #dpP,
        );
        u_var = vertcat(Fb,Tr);

        L = -((pP)*xp*(Fa+Fb)+pE*xe*(Fa+Fb) - pA*Fa - pB*Fb); #+dpP

        nx = np.prod(x_var.shape)
        nu = np.prod(u_var.shape)

        self.m |= {
            't': DM([]),
            'lbu': DM([0.1, 273]),
            'ubu': DM([20.0, 473]),
            'h': DM([]),
            'x': x_var,
            'lbx': DM.zeros(nx),
            'ubx': 1e5*DM.ones(nx),
            'f': diff,
            'z': DM([]),
            'lbz': DM([]),
            'ubz': DM([]),
            'u': u_var,
            'p': d_var,
            'y': vertcat(Fb, Tr, xg, xa), # vertcat(xb/xa), 
            # 'y': vertcat(Fb, Tr, xg, xc, xp, xe), # bad performance
            # 'y': vertcat(Fb, Tr, xg, xa, xc, xp), # bad 2
            # 'y': vertcat(Fb, Tr, xg, xa, xp, xe), # bad 3
            # 'y': vertcat(Fb, Tr, xg, xa, xb, xc), # bad almost
            # 'y': vertcat(Fb, Tr, xg, xe, xb, xc), # nao
            'g': 
            # DM([]),
            vertcat(
                # xa - 0.13,
                # 0.362 - xb,
                Fb - 4.0,
                Tr - 355,
                xg - 0.105,
                # -Fb,
                # 355 - Tr,
                # 0.11 - xp,
            ),
            'J': L
        }

    def guess(self, d):
        nx = np.prod(self.m['x'].shape)
        nz = np.prod(self.m['z'].shape)
        nu = np.prod(self.m['u'].shape)

        x0  = 0.5*DM.ones(nx)
        lbx = 1e-6*DM.ones(nx)
        ubx = 2.0*DM.ones(nx)
        # lbx = self.m['lbx']
        # ubx = self.m['ubx']
        u0  = DM([2.0, 373])
        lbu = DM([1.3, 338.]) # [0.5, 273]
        ubu = DM([10.0, 473])
        # lbu = self.m['ubu']
        # ubu = self.m['ubu']
        z0  = 0.5*DM.ones(nz)
        lbz = self.m['lbz']
        ubz = self.m['ubz']

        return u0, lbu, ubu, x0, lbx, ubx, z0, lbz, ubz